package org.nzdl.gsdl.GsdlCollageApplet;

import java.awt.*;
import java.io.*;
import java.net.*;
import java.util.*;

/** 
 *  @author Katrina Edgar
 *  @author David Bainbridge
 *
 *  Stores images that are downloaded but have not yet been displayed
 *  in the applet. Provides methods to add to and remove from this list
 *  and check it's size. It uses an inner class to store the data for these
 *  images, comprising of three parts: the image itself, it's source url
 *  and the name of the image */
public class DownloadImages {

    /** Data structure to store an image that has been downloaded and is
     *  waiting to be used. */
    public class ImageUrlTriple {

	Image image_;
	String url_;
	String name_;

	/** Constructs a downloaded image structure from the given parameters
	 *  @param image the graphical image
	 *  @param url the source url for this image
	 *  @param name the name of the image file */
	public ImageUrlTriple (Image image, String url, String name)
	{
	    image_ = image;
	    url_ = url;
	    name_ = name;
	}

	Image image() {
	    return image_;
	}

	String url() {
	    return url_;
	}

	String name() {
	    return name_;
	}

    }

    /** The amount of output to produce to the screen */
    int verbosity_       = 0;
    /** Holds the images that have been downloaded and are waiting to be used */
    Vector image_array_  = null;

    /** Initialises an empty vector to store downloaded images in and sets
     *  the verbosity for this class */
    public DownloadImages(int verbosity)
    {
	verbosity_ = verbosity;
	image_array_ = new Vector();
    }

    /** Get the number of images currently downloaded and waiting for use
     *  @return the number of images currently downloaded */
    public synchronized int size()
    {
	return image_array_.size();
    }

   //   /** Get the image from the specified position in the array of downloaded images
//       *  @return an ImageUrlTriple containing the image, url and image name */
//      public synchronized ImageUrlTriple get(int pos)
//      {
//  	return (ImageUrlTriple)image_array_.get(pos);
//      }

    /** Push a new image into the vector of downloaded images
     *  @param image the graphical image to add
     *  @param url the source url of the image
     *  @param name the name of the image to add */
    public synchronized void push(Image image, String url, String name)
    {
	ImageUrlTriple iutriple = new ImageUrlTriple(image,url,name);

	image_array_.addElement(iutriple);
    }

    /** Remove the image from the specified position in the array of downloaded images
     *  @return an ImageUrlTriple containing the image, url and image name */
    public synchronized ImageUrlTriple retrieve(int pos)
    {
	if (image_array_.size() == 0)
	    return null;

	if (pos >= (image_array_.size() - 1))
	    pos = 0;

	ImageUrlTriple u = (ImageUrlTriple) image_array_.elementAt(pos);
	image_array_.removeElementAt(pos);
	return u;
    }

    /** gets the oldest downloaded image, checks that the downloaded images are
     *  not empty.
     *  @return an ImageUrlTriple containing the image, url and image name */
    public synchronized ImageUrlTriple shift()
    {	
	if (image_array_.size()==0) {
	    return null;
	}

	ImageUrlTriple u = (ImageUrlTriple) image_array_.elementAt(0);
	image_array_.removeElementAt(0);
	return u;
    }
}
